﻿using System;
using System.Reflection;
using System.Runtime.InteropServices;

namespace CSharpConsole
{
    class Program
    {
        [DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern IntPtr GetModuleHandle(string lpModuleName);

        [DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern int GetModuleHandleEx(int dwFlags, int funcAddress, out IntPtr phModule);

        const int GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS = 0x4;

        static void Main(string[] args)
        {
            IntPtr pModule = GetModuleHandle(null);
            Console.WriteLine("GetModuleHandle(null) == 0x" + pModule.ToInt64().ToString("x"));

            //Action t = () => { };
            //IntPtr tAddress = Marshal.GetFunctionPointerForDelegate(t);

            MethodInfo mainMethod = typeof(Program).GetMethod("Main", BindingFlags.Static | BindingFlags.NonPublic);
            OutputModuleAddress(mainMethod);

            MethodInfo getCurrentThreadIdMethod = typeof(AppDomain).GetMethod("GetCurrentThreadId", BindingFlags.Static | BindingFlags.Public);
            OutputModuleAddress(getCurrentThreadIdMethod);

            Console.ReadLine();
        }

        private static void OutputModuleAddress(MethodInfo mi)
        {
            IntPtr tAddress = mi.MethodHandle.GetFunctionPointer();
            IntPtr pModule = IntPtr.Zero;

            int result = GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS, tAddress.ToInt32(), out pModule);
            int lastError = Marshal.GetLastWin32Error();

            if (lastError != 0 && lastError != 127)
            {
                Console.WriteLine(mi.Name + ": No Result(0x" + tAddress.ToInt32().ToString("x") + "), lastError = " + lastError + ", return = " + pModule.ToInt32().ToString("x"));
                return;
            }

            Console.WriteLine(mi.Name + " in 0x" + pModule.ToInt32().ToString("x"));
        }
    }
}
